@@ -8,7 +8,7 @@ use base qw{Module::Build};
 
 use autodie;
 use Cwd;
-use Path::Class qw{dir};
+use Path::Tiny;
 
 
 =head1 Methods and method modifiers
@@ -43,7 +43,7 @@ sub ACTION_build {
 
     $self->SUPER::ACTION_build(@args);
 
-    my $blibdir = dir()->parent->subdir('blib');
+    my $blibdir = path(getcwd)->child('blib');
     say "blibdir: $blibdir";
     $self->run_in_po_dir(qw{make install}, "DESTDIR=$blibdir");
 };
@@ -58,9 +58,9 @@ sub ACTION_test {
     my $self = shift;
     my @args = @_;
 
-    my $datadir = dir()->subdir('t')->subdir('data');
+    my $datadir = path(getcwd)->child('t', 'data');
     my @gnupg_homedirs = map {
-        $datadir->subdir("gnupg_homedir$_")->stringify
+        $datadir->child("gnupg_homedir$_")->stringify
     } ('', '_without_defined_keyserver', '_without_gpgconf');
     chmod 0700, @gnupg_homedirs;
 
