@@ -12,12 +12,15 @@ use English qw/-no_match_vars/;
 use File::HomeDir;
 use List::MoreUtils qw/uniq/;
 use List::Util qw/shuffle/;
-use Path::Class;
+use Path::Tiny;
 use Try::Tiny;
 
 use Exporter;
 our @ISA = qw(Exporter);
 
+use namespace::clean;
+
+
 =head1 NAME
 
 App::Parcimonie - tools for the parcimonie program
@@ -154,10 +157,10 @@ sub checkGpgHasDefinedKeyserver {
 
     my $gpgconf;
     if (defined $gnupg_homedir) {
-        $gpgconf = file($gnupg_homedir, 'gpg.conf');
+        $gpgconf = path($gnupg_homedir, 'gpg.conf');
     }
     else {
-        $gpgconf = file(File::HomeDir->my_home, '.gnupg', 'gpg.conf');
+        $gpgconf = path(File::HomeDir->my_home, '.gnupg', 'gpg.conf');
     }
 
     -f $gpgconf
