@@ -12,27 +12,22 @@ package App::Parcimonie::Daemon;
 use Moo;
 use MooX::late;
 
-with 'App::Parcimonie::Role::HasEncoding';
-use MooX::Options;
-
 use 5.10.1;
-use namespace::autoclean;
 use App::Parcimonie;
-use App::Parcimonie::DBus::Object;
+use Carp;
 use Encode;
 use English qw{-no_match_vars};
-use File::Spec;
-use List::MoreUtils qw{any};
-use Net::DBus;
-use Net::DBus::Reactor;
-use Net::DBus::Service;
-use Net::DBus::Test::MockObject;
-use Tie::Cache;
+use List::Util qw{any};
 use Time::Duration::Parse qw(parse_duration);
 use Try::Tiny;
 use Type::Utils qw{declare as where coerce from};
 use Types::Path::Tiny qw{Dir};
 use Types::Standard qw{Str Num};
+use namespace::clean;
+
+with 'App::Parcimonie::Role::HasEncoding';
+use MooX::Options;
+
 
 =head1 TYPES
 
@@ -50,6 +45,11 @@ option 'verbose'  => (
     isa => 'Bool', is => 'ro', default => 0
 );
 
+option 'with_dbus'  => (
+    documentation => q{Send activity updates on the org.parcimonie.daemon D-Bus service},
+    isa => 'Bool', is => 'ro', default => 0
+);
+
 option 'gnupg_homedir' => (
     documentation => q{GnuPG homedir.},
     isa           => Dir,
@@ -92,17 +92,6 @@ option 'minimum_lapse_time' => (
     coerce        => $DurationInSeconds->coercion,
 );
 
-has 'last_try' => (
-    documentation => q{In-memory LRU buffer storing the result of last key fetch attempts},
-    isa           => 'HashRef',
-    is            => 'rw',
-    default       => sub {
-        my %last_try;
-        tie %last_try, 'Tie::Cache', { MaxCount => 1000 };
-        return \%last_try;
-    },
-);
-
 has 'iterate_id' => (
     isa           => 'Int',
     is            => 'rw',
@@ -110,20 +99,8 @@ has 'iterate_id' => (
 
 has 'dbus_object' => (
     isa           => 'Object',
-    is            => 'rw',
-    default       => sub {
-        my ($bus, $service);
-        if ($ENV{HARNESS_ACTIVE}) {
-            $bus = Net::DBus->test;
-            $service = $bus->export_service("org.parcimonie.daemon");
-            Net::DBus::Test::MockObject->new($service, "/org/parcimonie/daemon/object");
-        }
-        else {
-            $bus = Net::DBus->session;
-            $service = $bus->export_service("org.parcimonie.daemon");
-            App::Parcimonie::DBus::Object->new($service);
-        }
-    },
+    is            => 'ro',
+    lazy_build    => 1,
 );
 
 option 'gnupg_already_torified' => (
@@ -134,6 +111,12 @@ option 'gnupg_already_torified' => (
     default   => sub { 0; },
 );
 
+has 'memory_usage' => (
+    isa       => 'Memory::Usage',
+    is        => 'ro',
+    required  => 0,
+);
+
 
 =head1 METHODS
 
@@ -149,6 +132,32 @@ sub BUILD {
 
     $self->keyserver_defined_on_command_line
         or checkGpgHasDefinedKeyserver($self->gnupg_options);
+
+    if ($self->with_dbus) {
+        require App::Parcimonie::DBus::Object;
+        require Net::DBus;
+        require Net::DBus::Reactor;
+        require Net::DBus::Service;
+        require Net::DBus::Test::MockObject;
+    }
+}
+
+sub _build_dbus_object {
+    my $self = shift;
+
+    croak "D-Bus is disabled." unless $self->with_dbus;
+
+    my ($bus, $service);
+    if ($ENV{HARNESS_ACTIVE}) {
+        $bus = Net::DBus->test;
+        $service = $bus->export_service("org.parcimonie.daemon");
+        Net::DBus::Test::MockObject->new($service, "/org/parcimonie/daemon/object");
+    }
+    else {
+        $bus = Net::DBus->session;
+        $service = $bus->export_service("org.parcimonie.daemon");
+        App::Parcimonie::DBus::Object->new($service);
+    }
 }
 
 =head2 run
@@ -161,6 +170,33 @@ sub run {
     my $opt  = shift;
     my $args = shift;
 
+    if ($self->with_dbus) {
+        $self->run_with_dbus();
+    }
+    else {
+        use JSON::PP;
+        $self->run_without_dbus();
+    }
+}
+
+sub run_without_dbus {
+    my $self = shift;
+
+    while (1) {
+        my $next_sleep_time = $self->iterate;
+        say encode_json({
+            state   => 'Sleeping',
+            details => {
+                duration => $next_sleep_time,
+            },
+        });
+        sleep($next_sleep_time);
+    }
+}
+
+sub run_with_dbus {
+    my $self = shift;
+
     my $reactor = Net::DBus::Reactor->main();
 
     my $initial_sleep_time = 1 * 1000;
@@ -178,6 +214,10 @@ sub run {
                         "Will now sleep %i seconds.",
                         $next_sleep_time
                     ));
+                    $self->notify({
+                        signal => 'Sleeping',
+                        duration => $next_sleep_time,
+                    });
                     $reactor->toggle_timeout(
                         $self->iterate_id,
                         1,
@@ -230,6 +270,38 @@ sub keyserver_defined_on_command_line {
     } @{$self->gnupg_extra_args};
 }
 
+sub notify {
+    my $self = shift;
+    my $args = shift;
+
+    if ($self->with_dbus) {
+        my $signal = $args->{signal};
+        my @args_name;
+        if ($signal eq 'FetchBegin') {
+            @args_name = qw{keyid};
+        }
+        elsif ($signal eq 'FetchEnd') {
+            @args_name = qw{keyid success gpg_error};
+        }
+        elsif ($signal eq 'Sleeping') {
+            @args_name = qw{duration};
+        }
+        else {
+            croak "Unsupported signal: $signal";
+        }
+
+        my @args;
+        push @args, $args->{$_} for (@args_name);
+        $self->dbus_object->emit_signal($signal, @args);
+    }
+    else {
+        say encode_json {
+            state   => $args->{signal},
+            details => $args,
+        };
+    }
+}
+
 sub tryRecvKey {
     my $self  = shift;
     my $keyid = shift;
@@ -237,7 +309,7 @@ sub tryRecvKey {
     my $gpg_error;
 
     $self->debug(sprintf("tryRecvKey: trying to fetch %s", $keyid));
-    $self->dbus_object->emit_signal("FetchBegin", $keyid);
+    $self->notify({ signal => 'FetchBegin', keyid => $keyid });
 
     try {
         $gpg_output = gpgRecvKeys(
@@ -260,9 +332,12 @@ sub tryRecvKey {
         $gpg_error = '';
     }
 
-    $self->set_last_try( $keyid => { success => $success, msg => $gpg_error } );
-    $self->dbus_object->emit_signal("FetchEnd", $keyid, $success, $gpg_error);
-
+    $self->notify({
+        signal    => 'FetchEnd',
+        keyid     => $keyid,
+        success   => $success,
+        gpg_error => $gpg_error,
+    });
 }
 
 sub next_sleep_time {
@@ -273,13 +348,17 @@ sub next_sleep_time {
             $self->average_lapse_time
             : averageLapseTime($num_public_keys);
 
-    $self->debug(sprintf('Using %s seconds as average sleep time.',
-                         $average_lapse_time
+    my $fallback_lapse_time = $self->minimum_lapse_time
+                            + rand($self->minimum_lapse_time);
+
+    $self->debug(sprintf('Using %s seconds as average sleep time, '.
+                         'and %s seconds as fallback sleep time.',
+                         $average_lapse_time, $fallback_lapse_time
     ));
 
     my $next_sleep_time = rand(2 * $average_lapse_time);
     if ($next_sleep_time < $self->minimum_lapse_time) {
-        $next_sleep_time = $self->minimum_lapse_time;
+        $next_sleep_time = $fallback_lapse_time;
     }
     return $next_sleep_time;
 }
@@ -293,6 +372,9 @@ Returns next sleep time (seconds).
 sub iterate {
     my $self = shift;
     my $default_sleep_time = 600;
+
+    $self->record_memory_usage("[iterate]: entering");
+
     my @public_keys = gpgPublicKeys(
         $self->gnupg_options,
         already_torified => $self->gnupg_already_torified,
@@ -306,17 +388,29 @@ sub iterate {
     # allow the GC to free some memory
     undef(@public_keys);
 
+    $self->record_memory_usage("[iterate]: before fetching key $keyid");
     $self->tryRecvKey($keyid);
+    $self->record_memory_usage("[iterate]: after fetching key $keyid");
 
     return $next_sleep_time;
 }
 
-sub set_last_try {
-    my $self   = shift;
-    my $keyid  = shift;
-    my $result = shift;
+=head2 record_memory_usage
 
-    $self->last_try->{$keyid} = $result;
+Args: a message (Str) indicating when we're run.
+Records memory usage in the memory_usage attribute,
+iff. the REPORT_MEMORY_USAGE environment variable is set to a true value.
+
+=cut
+sub record_memory_usage {
+    my $self = shift;
+    my $message = shift;
+
+    return unless exists $ENV{REPORT_MEMORY_USAGE}
+        && defined $ENV{REPORT_MEMORY_USAGE}
+        && $ENV{REPORT_MEMORY_USAGE};
+
+    $self->memory_usage->record($message);
 }
 
 no Moo;
