@@ -9,14 +9,16 @@ Have a look to App::Parcimonie::Daemon for a full-blown real life usage example.
 =cut
 
 package App::Parcimonie::GnuPG::Interface;
+
 use Moo;
 use MooX::late;
-extends 'GnuPG::Interface';
-
-use namespace::autoclean;
 use Carp;
 use File::Which qw{which};
 
+use namespace::clean;
+
+extends 'GnuPG::Interface';
+
 has 'already_torified' => (
     isa      => 'Bool',
     is       => 'rw',
